/*From OS 1.00 to OS 1.02
- OS checksum at 0x26FFF8
- Calculated as sum of 0x10000..0x26FFF7

From OS 1.03 to OS 1.11
- OS checksum at 0x26FFF8
- Calculated as sum of 0x10000..0x22FFFF + 0x260000..0x26FFF7

From OS 2.00 to OS 2.02
- OS checksum at 0x24FFF8
- Calculated as sum of 0x10000..0x24FFF7

From OS 2.00 to OS 2.02       (fx-7400GII)
- OS checksum at 0x1DFFF8
- Calculated as sum of 0x10000..0x1DFFF7
*/

#define CKOFFSET3	0x26FFF8 // fx-7400Gii OS checksum 1.xx
#define CKOFFSET2	0x24FFF8 // fx-9750/9860G OS checksum 2.xx
#define CKOFFSET1	0x1DFFF8 // fx-7400Gii OS checksum
#define VEROFFSET	0x010020
#define SHOFFSET	0x010080
#define SH3_ID		0xD0
#define SH4_ID		0xDF
#define ROM_SIZE1	(2*1024*1024)
#define ROM_SIZE2	2555904
typedef unsigned char byte;

#include "stdio.h"
#include "string.h"
#include "stdlib.h"

int getdigit(char c) {
	if(c>='0' && c<='9') return c-'0';
	return 0;
}

int isdigit(char c) { return c>='0' && c<='9'; }

void ckmod(byte* ptr, byte val, int& dck)
{	dck-=*ptr;
	*ptr=val;
	dck+=val;
}

int main(int argc, char* argv[])
{	byte* buffer = (byte*) malloc(ROM_SIZE2*sizeof(byte));
	int romsize,dck=0;
	FILE* rom;
	FILE* polyrom;

	if( argc<3 )
	{	printf("\n+----------------------------------+\n");
		printf(  "!            PolyOS 1.02           !\n");
		printf(  "+----------------------------------+\n");
		printf(" X. Andreani\n\n");
		printf("Modifies a Casio graphic calculator OS\n");
		printf("to make it installable on similar models.\n\n");
		printf("Supported models:\n");
		printf("* Graph 25+Pro / fx-7400Gii\n");
		printf("* Graph 35+USB / fx-9750Gii\n");
		printf("* Graph 85(SD) / fx-9860G(SD)\n");
		printf("* Graph 75/95 / fx-9860Gii(SD)\n");
		printf("ERROR: too few arguments\n");
		printf("Usage: %s <source_os> <dest_os>\n", argv[0]);
		return 0;
	}

	memset(buffer,0xFF,ROM_SIZE2);

	rom=fopen(argv[1],"rb");
	if(!rom)
	{	printf("Source file open error\n");
		return 0;
	}
	polyrom=fopen(argv[2],"wb");
	if(!polyrom)
	{	fclose(rom);
		printf("Dest file open error\n");
		return 0;
	}
	fseek(rom,0,SEEK_END);
	romsize=ftell(rom);
	fseek(rom,0,SEEK_SET);
	if(romsize>ROM_SIZE2) romsize=ROM_SIZE2;
	printf("File size is %i.\n",romsize);
	printf("Assuming model: fx-");
	if(romsize<=ROM_SIZE1)	printf("7400Gii");
	else					printf("9750/9860G");
	printf("\n");
	printf("Buffering file data...\n");
	int size=0;
	while(size<romsize)
	{	size+=fread(buffer+size,sizeof(byte),romsize-size,rom);
	}
	fclose(rom);
	if(romsize<=ROM_SIZE1)	romsize=ROM_SIZE1;
	else					romsize=ROM_SIZE2;
	printf("Assuming CPU  : ");
	if(buffer[SHOFFSET]==SH3_ID) printf("SH3");
	else if(buffer[SHOFFSET]==SH4_ID) printf("SH4");
	else printf("???");
	printf("\n");
	unsigned long version=0;
	int i;
	for(i=0;i<10;i++) {
		if(i!=2 && i!=5) {
			version*=10;
			version+=getdigit(buffer[VEROFFSET+i]);
		}
	}
	printf("Checking version ID: %d\n",version);
	printf("Version string is %c%c.%c%c.%c%c%c%c\n",buffer[VEROFFSET],buffer[VEROFFSET+1],buffer[VEROFFSET+3],buffer[VEROFFSET+4],buffer[VEROFFSET+6],buffer[VEROFFSET+7],buffer[VEROFFSET+8],buffer[VEROFFSET+9]);
	if(	version<1020000) {
		printf("Version string is too low or missing - patched to 01.02.0000\n");
		ckmod(buffer+VEROFFSET  ,'0',dck);
		ckmod(buffer+VEROFFSET+1,'1',dck);
		ckmod(buffer+VEROFFSET+2,'.',dck);
		ckmod(buffer+VEROFFSET+3,'0',dck);
		ckmod(buffer+VEROFFSET+4,'2',dck);
		ckmod(buffer+VEROFFSET+5,'.',dck);
		ckmod(buffer+VEROFFSET+6,'0',dck);
		ckmod(buffer+VEROFFSET+7,'0',dck);
		ckmod(buffer+VEROFFSET+8,'0',dck);
		ckmod(buffer+VEROFFSET+9,'0',dck);
	}
	else if(version>=2030000) {
		printf("Version string is too high - patched to ");
		if(buffer[SHOFFSET]==SH3_ID) {
			printf("02.01.XXXX\n");
			ckmod(buffer+VEROFFSET  ,'0',dck);
			ckmod(buffer+VEROFFSET+1,'2',dck);
			ckmod(buffer+VEROFFSET+3,'0',dck);
			ckmod(buffer+VEROFFSET+4,'1',dck);
		}
		else {
			printf("02.02.XXXX\n");
			ckmod(buffer+VEROFFSET  ,'0',dck);
			ckmod(buffer+VEROFFSET+1,'2',dck);
			ckmod(buffer+VEROFFSET+3,'0',dck);
			ckmod(buffer+VEROFFSET+4,'2',dck);
		}
	}
	printf("Removing specific model type...\n");
	memset(buffer+0x0000FFD0,255,8);

	byte code[] = { 0x4F,0x22,0xB0,0xFF,0x00,0x09,0x88,0x01,0x8B,0xFF,0xB0,0xFF,0x00,0x09,0x88,0x01,0x8B,0xFF,0xB0,0xFF,0x00,0x09,0x88,0x01 };
	byte check[]= {    1,   1,   1,   0,   1,   1,   1,   1,   1,   0,   1,   0,   1,   1,   1,   1,   1,   0,   1,   0,   1,   1,   1,   1 };
	printf("Looking for type checking code in the OS...\n");
	int found=0;
	for(i=0x10000; i<size && !found; i++)
	{	int bad=0;
		for(int j=0; j<24 && !bad; j++)
			if(buffer[i+j]!=code[j] && check[j]) bad=1;
		if(bad==0) found=1;
	}
	i--;
	if(found==0)
		printf("No such code found.\n");
	else
	{	byte* ptr=buffer+i;
		printf("Type checking code found at offset 0x%x!\n",i);
		printf("Killing type checking code...\n");
		ckmod(ptr+8,0x00,dck); 
		ckmod(ptr+9,0x09,dck); 
		ckmod(ptr+16,0x00,dck); 
		ckmod(ptr+17,0x09,dck); 
	}
	unsigned long cksum=0;
	long tcksum;
	int ckoffset=CKOFFSET2;
	if(version<2000000) ckoffset=CKOFFSET3;
	if(romsize<=ROM_SIZE1) ckoffset=CKOFFSET1;
	for(i=0;i<4;i++)
		cksum=cksum*256+buffer[i+ckoffset];
	printf("Original checksum at %x is 0x%x.\n",ckoffset,cksum);
	if(dck)
	{	printf("Adjusting checksum of %i...\n",dck);
		cksum+=dck;
		while(cksum<0) cksum+=0x100000000;
		while(cksum>0x100000000) cksum-=0x100000000;
		printf("New checksum is 0x%x.\n",cksum);
		tcksum=cksum;
		for(i=3;i>=0;i--)
		{	buffer[i+ckoffset]=tcksum%256;
			tcksum=tcksum/256;
		}
	}
	tcksum=cksum;
	if(romsize<=ROM_SIZE1)
	{	for(i=0;i<4;i++)
		{	cksum+=tcksum%256;
			tcksum=tcksum/256;
		}
		cksum+=0x6F8F808;
		printf("Adding a 2nd checksum for fx-9750/9860G compatibility of 0x%x.\n",cksum);
		for(i=3;i>=0;i--)
		{	buffer[i+CKOFFSET2]=cksum%256;
			cksum=cksum/256;
		}
	}
	printf("Flushing OS data...\n");
	size=0;
	while(size<ROM_SIZE2)
	{	size+=fwrite(buffer+size,sizeof(byte),ROM_SIZE2-size,polyrom);
	}
	fclose(polyrom);
	printf("Completed writing.\n");
	return 0;
}